<?php

  function i18n_search_index_init() {
    global $i18n_search_plugins, $i18n_search_index;
    $i18n_search_plugins = array();
    $i18n_search_index = array(
      'ids' => array(),
      'tags' => array(),
      'words' => array(),
      'dates' => array(),
      'fulltext' => array('tags' => I18N_TAG_WEIGHT, 'title' => I18N_TITLE_WEIGHT, 'content' => I18N_CONTENT_WEIGHT),
      'html' => array(),
      'fields' => array(),
      'noExtra' => array('id','language','tags','title','content','pubDate','creDate')
    );
    if (file_exists(GSDATAOTHERPATH.I18N_SEARCH_SETTINGS_FILE)) {
      $data = getXML(GSDATAOTHERPATH.I18N_SEARCH_SETTINGS_FILE);
      if (isset($data->contentWeight) && is_numeric((string) $data->contentWeight)) $i18n_search_index['fulltext']['content'] = (int) $data->contentWeight;
      if (isset($data->titleWeight) && is_numeric((string) $data->titleWeight)) $i18n_search_index['fulltext']['title'] = (int) $data->titleWeight;
      if (isset($data->tagWeight) && is_numeric((string) $data->tagWeight)) $i18n_search_index['fulltext']['tags'] = (int) $data->tagWeight;
      if (isset($data->fulltext)) {
        $contentWeight = $i18n_search_index['fulltext']['content'];
        foreach (preg_split("/[\s,]+/", trim($data->fulltext), -1, PREG_SPLIT_NO_EMPTY) as $field) {
          $pos = strpos($field,':');
          $weight = $pos > 0 ? intval(substr($field,$pos+1)) : $contentWeight;
          $i18n_search_index['fulltext'][$pos > 0 ? substr($field,0,$pos) : $field] = abs($weight);
          if ($weight < 0) $i18n_search_index['html'][] = $pos > 0 ? substr($field,0,$pos) : $field; 
        }
      }
    }
  }

  function i18n_search_register_plugin($pluginName) {
  }

  function i18n_search_index_item($pluginName, $data) {
    global $i18n_search_plugins, $i18n_search_index;
    $ismb = function_exists('mb_ereg_search');
    $id = $data['id'];
    $language = @$data['language'] ? $data['language'] : null;
    $fullId = $language ? $id.'_'.$language : $id; 
    $ids = &$i18n_search_index['ids'];
    if (!isset($ids[$id])) $ids[$id] = array(); 
    if ($language) $ids[$id][] = $language;
    $pubDate = @$data['pubDate'] ? $data['pubDate'] : 1;
    $creDate = @$data['creDate'];
    $i18n_search_index['dates'][$fullId] = $pubDate.' '.$creDate;
    if (@$data['tags']) {
      foreach (preg_split("/\s*,\s*/", trim($data['tags']), -1, PREG_SPLIT_NO_EMPTY) as $tag) {
        if ($ismb) {
          $tag = mb_ereg_replace("[^\w]", "_", mb_strtolower($tag, 'UTF-8'));
        } else {
          $tag = preg_replace("/[^\w]/", "_", strtolower($tag));
        }
        $tags = &$i18n_search_index['tags'];
        if (!isset($tags[$tag])) $tags[$tag] = array($id => 1); else $tags[$tag][$id] = 1;
      }
    }
    $words = &$i18n_search_index['words'];
    foreach ($data as $field => $value) {
      $weight = intval(@$i18n_search_index['fulltext'][$field]);
      $isExtra = !in_array($field, $i18n_search_index['noExtra']);
      if ($weight > 0 || $isExtra) {
        if ($ismb) {
          mb_ereg_search_init($value, "\w+");
          if (mb_ereg_search()) {
            $match = mb_ereg_search_getregs();
            do {
              $word = mb_strtolower($match[0], 'UTF-8');
              if ($weight) {
                if (!isset($words[$word])) $words[$word] = array();
                $words[$word][$fullId] = (isset($words[$word][$fullId]) ? $words[$word][$fullId] : 0) + $weight;
              }
              if ($isExtra) {
                if (!isset($i18n_search_index['fields'][$field])) $i18n_search_index['fields'][$field] = array();
                $fieldwords = $i18n_search_index['fields'][$field];
                if (!isset($fieldwords[$word])) $fieldwords[$word] = array();
                $fieldwords[$word][$fullId] = (isset($fieldwords[$word][$fullId]) ? $fieldwords[$word][$fullId] : 0) + $weight;
              }
              $match = mb_ereg_search_regs();
            } while ($match);
          }
        } else {
          preg_match_all("/\w+/", $content, $matches);
          foreach ($matches[0] as $word) {
            $word = strtolower($word);
            if ($weight) {
              if (!isset($words[$word])) $words[$word] = array();
              $words[$word][$fullId] = (isset($words[$word][$fullId]) ? $words[$word][$fullId] : 0) + $weight;
            }
            if ($isExtra) {
              if (!isset($i18n_search_index['fields'][$field])) $i18n_search_index['fields'][$field] = array();
              $fieldwords = $i18n_search_index['fields'][$field];
              if (!isset($fieldwords[$word])) $fieldwords[$word] = array();
              $fieldwords[$word][$fullId] = (isset($fieldwords[$word][$fullId]) ? $fieldwords[$word][$fullId] : 0) + $weight;
            }
          }
        }
      }  
    }
  }

  $contentWeight = I18N_CONTENT_WEIGHT;
  $titleWeight = I18N_TITLE_WEIGHT;
  $tagWeight = I18N_TAG_WEIGHT;
  if (file_exists(GSDATAOTHERPATH.I18N_SEARCH_SETTINGS_FILE)) {
    $data = getXML(GSDATAOTHERPATH.I18N_SEARCH_SETTINGS_FILE);
    if (isset($data->contentWeight) && is_numeric((string) $data->contentWeight)) $contentWeight = (int) $data->contentWeight;
    if (isset($data->titleWeight) && is_numeric((string) $data->titleWeight)) $titleWeight = (int) $data->titleWeight;
    if (isset($data->tagWeight) && is_numeric((string) $data->tagWeight)) $tagWeight = (int) $data->tagWeight;
  }

  $tags = array();
  $words = array();
  $dates = array();
  $defUrls = array();
  $nondefUrls = array();
  // use multibyte string functions?
  $ismb = function_exists('mb_ereg_search');
  if ($ismb) mb_regex_encoding('UTF-8');
  // read pages
	$dir_handle = @opendir(GSDATAPAGESPATH) or die("Unable to open pages directory");
	while ($filename = readdir($dir_handle)) {
    if (strrpos($filename,'.xml') === strlen($filename)-4 && !is_dir(GSDATAPAGESPATH . $filename) ) {
			$pagedata = getXML(GSDATAPAGESPATH . $filename);
      $private = (string) $pagedata->private;
      if ($private == 'Y') continue;
      $url = '' . $pagedata->url;
      $pubDate = strtotime((string) $pagedata->pubDate);
      $creDate = isset($pagedata->creDate) ? @strtotime((string) $pagedata->creDate) : $pubDate; 
      $dates[$url] = $pubDate . ' ' . $creDate;
      $metak = html_entity_decode(strip_tags(stripslashes(htmlspecialchars_decode($pagedata->meta))), ENT_QUOTES, 'UTF-8');
      $title = html_entity_decode(strip_tags(stripslashes(htmlspecialchars_decode($pagedata->title))), ENT_QUOTES, 'UTF-8');
      $content = html_entity_decode(strip_tags(stripslashes(htmlspecialchars_decode($pagedata->content))), ENT_QUOTES, 'UTF-8');
      if (strpos($filename,'_') === false) {
        $defUrls[$url] = array();
        foreach (preg_split("/\s*,\s*/", trim($metak), -1, PREG_SPLIT_NO_EMPTY) as $tag) {
          if ($ismb) {
            $tag = mb_ereg_replace("[^\w]", "_", mb_strtolower($tag, 'UTF-8'));
          } else {
            $tag = preg_replace("/[^\w]/", "_", strtolower($tag));
          }
          if (!isset($tags[$tag])) $tags[$tag] = array();
          $tags[$tag][$url] = 1;
          $defUrls[$url][] = $tag;
        }
      } else {
        $nondefUrls[] = $url;
      }
      if ($ismb) {
        mb_ereg_search_init($metak, "\w+");
        if (mb_ereg_search()) {
          $match = mb_ereg_search_getregs();
          do {
            $word = mb_strtolower($match[0], 'UTF-8');
            if (!isset($words[$word])) $words[$word] = array();
            $words[$word][$url] = (isset($words[$word][$url]) ? $words[$word][$url] : 0) + $tagWeight;
            $match = mb_ereg_search_regs();
          } while ($match);
        }
        mb_ereg_search_init($title, "\w+");
        if (mb_ereg_search()) {
          $match = mb_ereg_search_getregs();
          do {
            $word = mb_strtolower($match[0], 'UTF-8');
            if (!isset($words[$word])) $words[$word] = array();
            $words[$word][$url] = (isset($words[$word][$url]) ? $words[$word][$url] : 0) + $titleWeight;
            $match = mb_ereg_search_regs();
          } while ($match);
        }
        mb_ereg_search_init($content, "\w+");
        if (mb_ereg_search()) {
          $match = mb_ereg_search_getregs();
          do {
            $word = mb_strtolower($match[0], 'UTF-8');
            if (!isset($words[$word])) $words[$word] = array();
            $words[$word][$url] = (isset($words[$word][$url]) ? $words[$word][$url] : 0) + $contentWeight;
            $match = mb_ereg_search_regs();
          } while ($match);
        }
      } else {
        preg_match_all("/\w+/", $metak, $matches);
        foreach ($matches[0] as $word) {
          $word = strtolower($word);
          if (!isset($words[$word])) $words[$word] = array();
          $words[$word][$url] = (isset($words[$word][$url]) ? $words[$word][$url] : 0) + $tagWeight;
        }
        preg_match_all("/\w+/", $title, $matches);
        foreach ($matches[0] as $word) {
          $word = strtolower($word);
          if (!isset($words[$word])) $words[$word] = array();
          $words[$word][$url] = (isset($words[$word][$url]) ? $words[$word][$url] : 0) + $titleWeight;
        }
        preg_match_all("/\w+/", $content, $matches);
        foreach ($matches[0] as $word) {
          $word = strtolower($word);
          if (!isset($words[$word])) $words[$word] = array();
          $words[$word][$url] = (isset($words[$word][$url]) ? $words[$word][$url] : 0) + $contentWeight;
        }
      }
    }
  }
  // add non default language pages to tags
  foreach ($nondefUrls as $url) {
    $rawurl = substr($url,0,strpos($url,'_'));
    if (isset($defUrls[$rawurl])) {
      foreach ($defUrls[$rawurl] as $tag) {
        $tags[$tag][$url] = 1;
      }
    }
  } 
  // date file
  ksort($dates);
  $f = fopen(GSDATAOTHERPATH . I18N_DATE_INDEX, "w");
  foreach ($dates as $url => $date) {
    fputs($f, "$url $date\n");
  }
  fclose($f);
  // tag index file
  ksort($tags);
  $f = fopen(GSDATAOTHERPATH . I18N_TAG_INDEX, "w");
  foreach ($tags as $tag => $urls) {
    fputs($f, $tag);
    foreach ($urls as $url => $score) fputs($f, ' '.$url);
    fputs($f, "\n");
  }
  fclose($f);
  // word index file
  ksort($words);
  $f = fopen(GSDATAOTHERPATH . I18N_WORD_INDEX, "w");
  foreach ($words as $word => $urls) {
    fputs($f, $word);
    foreach ($urls as $url => $score) if ($score > 0) fputs($f, ' '.$url.':'.$score);
    fputs($f, "\n");
  }
  fclose($f);

